# This document contains the Shiny Modules for the ITHIM TOOL.

ithim_toolUI <- function(id){
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # The function NS() here is utilized
  # to "modularize" the Shiny code
  # below. While modules are typically
  # used for repetitive code where you
  # might want to reproduce that bit of
  # code multiple times; here, we're
  # using Shiny modules to make code
  # maintenance more simple by packing
  # away all the "tool" components into
  # a separate .R file. A tutorial on
  # Shiny modules can be found here: htt
  # ps://www.rstudio.com/resources/webin
  # ars/understanding-shiny-modules/
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ns <- NS(id)
  # return tagList
  tagList(
    fluidRow(
      class="ithimTool_title_div",
      tags$p(
        "ITHIM TOOL",
        class="ithimTool_title"
      ),
      includeCSS(file.path(pth, "/ithim_tool.css"))  # ITHIM Table CSS
      
    ),
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # ITHIM Tool Label Div ----
    fluidRow(
      style=paste(
        "background-color: ",
        otros_colores[4],
        "; ",
        "border-width: 1px; ",
        "border-style: solid; ",
        "border-color: ",
        otros_colores[6],
        "; ",
        "border-radius: 5px; ",
        "box-sizing: border-box; ",
        # "color:",
        # colores[5],
        # "; ",
        "margin: 1px 0.5em 5em 0.5em; ",
        "padding: 0px; ",
        "-webkit-box-shadow: inset 0 1px 0px white, ",
        "0 8px 6px -6px black; ",
        "-moz-box-shadow: inset 0 1px 0px white, ",
        "0 8px 6px -6px black; ",
        "box-shadow: inset 0 1px 0px white, ",
        "0 8px 6px -6px black; ",
        "overflow: hidden; "
      ),  # end paste
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Model Selection Panel ----
      column(
        width = 4,
        offset = 0,
        style = paste(
          "background-color: transparent; ",
          "color:",
          otros_colores[7],
          "; ",
          "margin: 0; ",
          "color: #1F8BBF; "
        ),
        tags$h3(
          "Make Your Selections",
          class="ithimTool_modelSelection_headers"
        ),
        tags$br(),
        uiOutput(ns("user_choice")),
        uiOutput(ns("user_input")),
        uiOutput(ns("user_base_select")),
        uiOutput(ns("user_scenario_select")),
        column(width=6,
               style=styles$minimal_padding,
               uiOutput(ns("absolute_time_walk"))
        ),
        column(width=6,
               style=styles$minimal_padding,
               uiOutput(ns("absolute_time_bike"))
        ),
        column(width=12,
               style=styles$minimal_padding,
               uiOutput(ns("multiples"))
        ),
        column(width=12,
               style=styles$minimal_padding,
               uiOutput(ns("percent_ev"))
        ), 
        column(width=12,
               style=styles$minimal_padding,
               uiOutput(ns("commentary"))),
        uiOutput(ns("year_select")),
        uiOutput(ns("geography_select")),
        uiOutput(ns("user_geography_select")),
       
        # horizontal rule
        tags$hr(class="hr_thin"),
        tags$h4("Output",
                class="ithimTool_modelSelection_headers"),
        uiOutput(ns("data_vis_select")),
        uiOutput(ns("detail_select")),
        uiOutput(ns("disease_select")),
        uiOutput(ns("units_centrality")),
        uiOutput(ns("units_active_travel")),
        uiOutput(ns("units_travel_dist")),
        uiOutput(ns("units_3_select")),
        # horizontal rules
        tags$br(),
        tags$hr(class="hr_thin_min_mar"),
        tags$hr(class="hr_thin_min_mar"),
        tags$hr(class="hr_thin_min_mar"),
        # tags$br(),
        uiOutput(ns("info_box")),
        tags$h4("Downloads", 
                class="ithimTool_modelSelection_headers"),
        uiOutput(ns('downloadData')),
        br(),
        br(),
        uiOutput(ns('downloadReport'))
      ),  # End of column
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Output Panel ----
      # Tables/Graphs Output - Main Panel 1/2
      column(width=8,
             offset=0,
             style=
               paste(
                 "background-color: white; ",
                 "color:", 
                 colores[5], 
                 "; "
               ),
             uiOutput(ns("data_integrity")),
             uiOutput(ns("msg")),
             uiOutput(ns("title")),
             uiOutput(ns("vis"))
      )  # end of column
    )  # End of fluidRow
  )  # end tagList
}

ithim_tool <- function(input, output, session){
  ns <- session$ns
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # rV: tracker ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Keeps track of different options. (Note: this is not the breadcrumbs.)
  tracker <- reactiveValues(
    ready_for_user = NULL,  # a switch (T/F)
    usr_narratives = NA,  # a list of user narratives
    usr_error = NA,
    msg = NA,
    # usr_baselines = NULL,  # names of the baselines/bau
    # usr_scenarios = NULL,  # names of the scenarios
    specific_disease = F,  # a switch that turns on optional spec dis panel.
    current_tagList = NULL,
    title_change = NA
  )
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Create list of options for default options selection widget
  # default_options <- reactive({
  #   l <- list()
  #   
  #   # Add default scenarios
  #   nm <- names(default_narratives)
  #   nm <- nm[!grepl("baseline", nm, ignore.case=T)]
  #   l <- append(l, nm)
  #   rm(nm)
  #   
  #   # Add other options: csmp/carb, whatif, absolute, user upload
  #   l <- append(l, c("CARB Scoping Plan (2030)", 
  #                    "Caltrans Strategic Management Plan (2020)", 
  #                    "Baseline Multiples",
  #                    "Fixed Time",
  #                    "Low Carbon Driving",
  #                    "User Upload"))
  #   
  #   l  # return
  # })
  
  # rx: isolate_geography ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Get regions/geographies for selectInput
  isolate_geography <- reactive({
    req(input$chosen_scenario)
    
    # Regions/geographies not specified by scenario
    if(input$chosen_scenario %in% 
#       c("CARB Scoping Plan (2030)", 
#         "Caltrans Strategic Management Plan (2020)", 
       c("Baseline Multiples",
         "Low Carbon Driving",
         "Fixed Time")){
      # get region from baseline
      regions <- unique(default_narratives$Baseline2015$Distances$Region)
      
    } else if(input$chosen_scenario=="User Upload") { 
      # specified by usr scenario
      req(input$chosen_user_baseline)
      req(input$chosen_user_scenario)
      
      # get regions from baseline and scenario.
      regions <-
        fn_pullRegions(
          list(tracker$usr_narratives[[input$chosen_user_baseline]],
               tracker$usr_narratives[[input$chosen_user_scenario]]))
      
      geographies <- d$regions[which(d$regions$Region %in% regions),
                               "County"]
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Here we *could* attempt to have a
      # sorting scheme like we do for
      # default scenarios, but instead we
      # currently have it so it just outputs
      # a vector of all the "Region"s
      # indicated by the user's csv.
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
      # out <- list()
      # 
      # if("California" %in% geographies) {
      #   out <- append(out, list(State="California"))
      # }
      # 
      # if(unique(d$regions[,"Region"]) %in% geographies){
      #   out <- append(out, list(Regions = regions[order(regions)]))
      # }
      # 
      # if(unique(d$))
      
      return(geographies)
      
    } else {  # other default scenarios
      regions <- fn_pullRegions(
        list(default_narratives$Baseline2015,
             default_narratives[[input$chosen_scenario]]))
    }
    
    geographies <- d$regions[which(d$regions$Region %in% regions),
                             "County"]
    
    # remove regions from geographies
    geographies <- geographies[which(!(geographies %in% regions))]
    
    # remove claifornia from regions
    regions <- regions[which(!(regions %in% "United States"))]
    
    out <- list(
      Geography = list("United States")
#      Regions = regions[order(regions)],
#      Counties = geographies[order(geographies)]
    )
    
    out  # return
  })
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # rx: run ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Generate ITHIM products. In other
  #' words: create a list with critical
  #' setup data, and run through all the
  #' core ITHIM functions.
  #' 
  #' @param input$chosen_scenario
  #' @param input$chosen_geography
  #' @param input$chosen_year
  #' @return l list
  run <- reactive({
    req(input$chosen_scenario)
    req(input$chosen_visual)
    req(input$chosen_year)
    
    if(T %in% grepl(pattern="Fatal Error", x=d$data_integrity))
      return(NULL)
    
    if(input$chosen_scenario == "User Upload") {
      req(input$chosen_user_scenario)
      # req(tracker$ready_to_run)
      req(input$chosen_user_geography)
      req(tracker$usr_narratives)
    } else {
      req(input$chosen_geography)
    }
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
    # Given the user selections, select
    # the appropriate 'baseline' and
    # 'scenario' data to run through the
    # model. Further, add a class to help
    # ITHIM functions determine what kind
    # of scenario it may be (eg. is it
    # default, user, multiples, or
    # Fixed Time). Finally, attach the
    # name of the narrative (eg. SCS 2040)
    # to the baseline and scenario.
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Note, scenario_type determines how
    # we manipulate the distances.
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    if(input$chosen_scenario == "User Upload"){  
      # SCENARIO: USER

      # BASELINE
      baseline <- tracker$usr_narratives[[input$chosen_user_baseline]]
      # attach the name of the baseline here.
      baseline <- append(baseline,
                         list(name=input$chosen_user_baseline))
      # give class
      attr(baseline, "scenario_type") <- "user_defined"  
      
      # SCENARIO
      scenario <- tracker$usr_narratives[[input$chosen_user_scenario]]
      # attach the name of the baseline here.
      scenario <- append(scenario,
                         list(name=input$chosen_user_scenario))
      # give class
      attr(scenario, "scenario_type") <- "user_defined"
      
      
    } else if(input$chosen_scenario == "Baseline Multiples"){
      # SCENARIO: MULTIPLES
      
      req(input$chosen_multiples_walk)
      req(input$chosen_multiples_bike)
      req(input$chosen_multiples_transit)
      
      baseline <- default_narratives$Baseline2015
      # attach the name of the baseline here.
      baseline <- append(baseline,
                         list(name="Baseline2015"))
      # give class
      attr(baseline, "scenario_type") <- "multiples"  
      
      # Create the scenario from the user supplied multiples.
      multiples <- c(input$chosen_multiples_walk,
                     input$chosen_multiples_bike,
                     input$chosen_multiples_transit)
      scenario <- list(Distances=NULL,
                       multiples=multiples,
                       name=paste0("Multiples_",
                                   paste0(multiples, collapse="-")))
      
      # Assign the appropriate class to the data.
      attr(scenario, "scenario_type") <- "multiples"
      
    } else if(input$chosen_scenario == "Low Carbon Driving") {
      # SCENARIO: LCD
      
      baseline <- default_narratives$Baseline2015
      # attach the name of the baseline here.
      baseline <- append(baseline,
                         list(name="Baseline2015"))
      # give class
#      attr(baseline, "scenario_type") <- "default"  
      
#      scenario <- list(Distances=NULL,
#                      name="Low Carbon Driving")

      # Assign the appropriate class to the data.
#      attr(scenario, "scenario_type") <- "default"
 
      # give class
      attr(baseline, "scenario_type") <- "evpct"  
      
      # Create the scenario from the user supplied EV percent.
      evpct <- c(input$chosen_pct_ev_sub)
  
      scenario <- list(Distances=NULL,
                       evpct=evpct,
                       name=paste0("Low Carbon Driving_",
                                   paste0(evpct, "%")))
      
      # Assign the appropriate class to the data.
      attr(scenario, "scenario_type") <- "evpct"      
           
    } else if (input$chosen_scenario %in%
               c("CARB Scoping Plan (2030)",
                 "Caltrans Strategic Management Plan (2020)")) {
      # SCENARIO: CARB or CSMP
      
      # A Variant of Multiples
      baseline <- default_narratives$Baseline2015
      # attach the name of the baseline here.
      baseline <- append(baseline,
                         list(name="Baseline2015"))
      # give class
      attr(baseline, "scenario_type") <- "multiples"  
      
      if(input$chosen_scenario == "CARB Scoping Plan (2030)") {
        scenario <- list(Distances=NULL,
                         multiples=c(4,9,4),
                         name="CARB Scoping Plan (2030)")
        
      } else if(input$chosen_scenario == 
                "Caltrans Strategic Management Plan (2020)"){
        scenario <- list(Distances=NULL,
                         multiples=c(2,3,2),
                         name="Caltrans Strategic Management Plan (2020)")
      }
      
      # Assign the appropriate class to the data.
      attr(scenario, "scenario_type") <- "multiples"
      
      # SCENARIO: FIXED TIME
    } else if(input$chosen_scenario == "Fixed Time"){  # Absolute data
      req(input$chosen_abs_walk)
      req(input$chosen_abs_bike)
      
      baseline <- default_narratives$Baseline2015
      baseline <- append(baseline,
                         list(name="Baseline2015"))  # name of the baseline.
      # give class
      attr(baseline, "scenario_type") <- "absolute_time"
      
      # Create the scenario from user supplied Fixed Times.
      a_time <- c(input$chosen_abs_walk,
                  input$chosen_abs_bike)
      scenario <- list(Distances=NULL,
                       absolute_time=a_time,
                       name=paste0("FixedTime_",  # name of the scenario.
                                   paste0(a_time, collapse="-")))
      
      # Assign the appropriate class to the data.
      attr(scenario, "scenario_type") <- "absolute_time"
      
      # SCENARIO: DEFAULT
    } else {  # Default scenarios
      baseline <- default_narratives$Baseline2015
      baseline <- append(baseline,
                         list(name="Baseline2015")) 
      attr(baseline, "scenario_type") <- "default"
      
      scenario <- default_narratives[[input$chosen_scenario]]
      # attach the name of the baseline here.
      scenario <- append(scenario,
                         list(name=input$chosen_scenario))
      # give class
      attr(scenario, "scenario_type") <- "default"
    }
    
    # Regions & Geographies
    if(input$chosen_scenario == "User Upload"){
      # Get Region, based on geography.
      geography <- input$chosen_user_geography
      region <- d$regions[which(d$regions$County==geography), 
                          "Region"]
    } else {
      # Get Region, based on geography.
      geography <- input$chosen_geography
      region <- d$regions[which(d$regions$County==geography), 
                          "Region"]
    }
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Further Selection Specific Modifications

    # p  - car miles substitution
    if(input$chosen_scenario == "Baseline Multiples"){
      req(input$chosen_car_mi_sub)
      p <- input$chosen_car_mi_sub/100
    } else {
      p <- 1
    }

    # EV percent substitution; must have value, even NA
    if(input$chosen_scenario == "Low Carbon Driving"){
      req(input$chosen_pct_ev_sub)
      EV_PCT <- input$chosen_pct_ev_sub 
    } else {
      EV_PCT <- NA
    }
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
    # Now, prepare a list object for the
    # model 'run' using the above baseline
    # and scenarios.
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    l <- list(
      baseline = baseline,  # the baseline created above.
      scenario = scenario,  # the scenario created above.
      fRegion = region,
      fGeography = geography,
      fYear = input$chosen_year,
      table_units = "default",  # regards footnotes for Tables.
      p = p,
      EV_PCT = EV_PCT,
      is_formatted = F
    )
    
    # Select Units Conversion
    if(input$chosen_visual %in% c("Tables", "Graphs")) {
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # TABLES & GRAPHS
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      req(input$chosen_units_centrality)
      req(input$chosen_units_active_travel_time)
      req(input$chosen_units_travel_dist_numerator)
      req(input$chosen_units_travel_dist_denominator)
      # Centrality: MEAN or MEDIAN
      l$centrality <- input$chosen_units_centrality
      # Active Travel: minutes per DAY or WEEK
      l$at_time_unit <- input$chosen_units_active_travel_time
      # Travel Distance: MILES or KILOMETERS
      l$td_unit <- input$chosen_units_travel_dist_numerator
      # Travel Distance: DAY, WEEK, or YEAR
      l$td_time <- input$chosen_units_travel_dist_denominator
      
    } else {
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # REPORT & INFOGRAPHIC
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Centrality: MEAN or MEDIAN
      l$centrality <- "Mean"
      # Active Travel: minutes per DAY or WEEK
      l$at_time_unit <- "Week"
      # Travel Distance: MILES or KILOMETERS
      l$td_unit <- "Miles"
      # Travel Distance: DAY, WEEK, or YEAR
      l$td_time <- "Week"
    }
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
    # Send the above list object through
    # all the core ITHIM functions. To see
    # what these functions are doing,
    # please open ithim_functions.R
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    l <- fn_prep(l)
    
    l <- fn_distances(l)
    
    l <- fn_scenario_time(l)
    
    l <- fn_pa_quantile(l)
    
    l <- fn_pa_paf(l)
    
    l <- fn_pm_paf(l)
    
    l <- fn_rti_paf(l)
    
    l <- fn_burden_causes(l)
    
    l <- fn_monetization(l)
    
    l <- fn_carbon_emissions(l)
    
    l <- fn_units_conversion(l)
    
    # saveRDS(l, "tmp")  # For code testing purposes only.
    
    l  # return
  })  # End of run()
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # oE: Scenario Changes ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Once User Upload selected, get the user upload feature prepared.
  observeEvent(input$chosen_scenario,
               {
                 # Changes if any Scenario is Selected
                 tracker$usr_narratives <- NULL  # clear usr_narratives.
                 tracker$msg <- NA  # clear messages.
                 
                 # Changes if User Upload Selected
                 if(input$chosen_scenario=="User Upload") {
                   tracker$ready_for_user <- T
                 }
                 
                 # tracker$ready_to_run <- NULL
                 # tracker$usr_baselines <- NULL
                 # tracker$usr_scenarios <- NULL
               }
  )
  
  # oE: User Narratives ---- 
  # Once uploaded, send values to tracker
  observeEvent(input$user_upload,
               if(nrow(input$user_upload)==1){
                 input_csv <- input$user_upload
                 
                 # if nothing, return nothing (so zen!)
                 if(is.null(input_csv)){
                   tracker$ready_for_user <- F
                   break() 
                 }
                 
                 # Now, actually read in the csv.
                 df <- read.csv(input_csv$datapath, stringsAsFactors = F)
                 
                 # Split scenarios by Item_Name
                 narratives <- fn_narrativeSplitter(df)
                 
                 # Verify user data
                 verified <- fn_user_data_verify(narratives)
                 
                 # Save any messages to tracker if present.
                 if(!(is.na(verified$msg)))
                   tracker$msg <- verified$msg
                 else
                   # Or just make it blank.
                   tracker$msg <- NA
                 
                 if(verified$continue == T){
                   # Passed verification.
                   
                   if(!(is.na(verified$msg)))
                     narratives <- fn_user_data_subs(narratives)
                   # browser()
                   tracker$usr_narratives <- narratives
                   tracker$ready_for_user <- F
                 } else {
                   # Failed to pass verification.
                   tracker$usr_narratives <- NULL
                   tracker$ready_for_user <- T  # reset user upload.
                 }
               }
  )
  
  # oE: Specific Disease ----
  observeEvent(input$chosen_detail,
               {
                 if(input$chosen_detail == "High") 
                   tracker$specific_disease <- T
                 else 
                   tracker$specific_disease <- F
               })
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # RENDER ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # Input: Main Choice ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates Primary Scenario selection drop down widget.
  output$user_choice <- renderUI(
    selectInput(
      inputId = ns("chosen_scenario"),
      label = "Scenarios",
      selected = "U. S. Surgeon General Recommendation",
      choices = d$main_choice[3:8],
      width="100%"
    )
  )
  
  # Input: Upload ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates User Upload Widget for Narrative CSV upload.
  output$user_input <- renderUI({
    req(input$chosen_scenario)
    req(tracker$ready_for_user)
    if(input$chosen_scenario=="User Upload" & 
       tracker$ready_for_user==T){
      tagList(
        fileInput(
          inputId = ns("user_upload"),
          label = "Select a file to upload.",
          multiple = F,
          accept = "text/csv"
        ),
        tags$br()
      )
    }
  }
  )
  
  # Print: User Msg ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates an error notice depending on if tracker$msg is present.
  output$msg <- renderUI({
    req(tracker$msg)
    if(!(is.na(tracker$msg)))
      tagList(
        tags$div(
          tags$p(HTML(tracker$msg)),
          class = "error_msg"
        )
      )
  })
  
  # Print: Data Integrity ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  output$data_integrity <- renderUI({
    if(length(d$data_integrity) > 0){
      tagList(
        tags$div(
          tags$p(paste(unlist(d$data_integrity))),
          class = "error_msg"
        )
      )
    }
  })
  
  # Input: User Baseline Select ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates User Uploaded Data Baseline selection drop down widget.
  output$user_base_select <- renderUI({
    req(input$chosen_scenario)
    req(tracker$usr_narratives)
    # req(tracker$usr_error)
    # req(user_scenarios)
    # req(tracker$usr_baselines)
    
    if (input$chosen_scenario == "User Upload") {
      # get names of narratives (i.e. scenarioID)
      nm <- names(tracker$usr_narratives)
      
      # get default baseline
      if(T %in% grepl("baseline", nm, ignore.case = T)){
        # get first narrative with "baseline" in title.
        sel <- nm[grepl("baseline", nm, ignore.case = T)][1]
      } else {
        # otherwise, just select first narrative.
        sel <- nm[1]
      }
      
      selectInput(
        inputId = ns("chosen_user_baseline"),
        label = "Please select a baseline.",
        selected = sel,
        choices = nm,
        width = "100%"
      )
    }
  })
  
  # Input: User Scenario Select ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates User Uploaded Data Scenario selection drop down widget.
  output$user_scenario_select <- renderUI({
    req(input$chosen_scenario)
    req(tracker$usr_narratives)
    # req(tracker$usr_error)
    # req(tracker$usr_scenarios)
    
    
    # if (!is.na(tracker$usr_error))
    #   NULL
    if (input$chosen_scenario == "User Upload") {
      nm <- names(tracker$usr_narratives)
      
      # get default scenario
      if(T %in% grepl("scenario", nm, ignore.case = T)){
        # get first narrative with "scenario" in title.
        sel <- nm[grepl("scenario", nm, ignore.case = T)][1]
      } else {
        # otherwise just get last narrative
        sel <- nm[length(nm)]
      }
      
      selectInput(
        inputId = ns("chosen_user_scenario"),
        label = "Please select a scenario.",
        selected = sel,
        choices = nm,
        width = "100%"
      )
    }
  })
  
  # Input: User Geography Select ----
  output$user_geography_select <- renderUI({
    req(input$chosen_user_scenario)
    req(input$chosen_user_baseline)
    req(tracker$usr_narratives)
    if (input$chosen_scenario == "User Upload" &
        !is.null(tracker$usr_narratives)
        ){
      selectInput(
        inputId = ns("chosen_user_geography"),
        label = "Please select a geography:",
        choices = isolate_geography(),
        width="100%"
      )
    }
  })
  
  # Input: Fixed Time Numeric ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates Fixed Time walk input box.
  output$absolute_time_walk <- renderUI({
    req(input$chosen_scenario)
    if(input$chosen_scenario=="Fixed Time"){
      numericInput(
        inputId = ns("chosen_abs_walk"),
        label = "Walk: ",
        value = 37,
        min = 0,
        max = 200,
        step = 0.5,
        width = "100%"
      )
    }
    
  })
  
  # Input: Fixed Time Numeric
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates Fixed Time bike input box.
  output$absolute_time_bike <- renderUI({
    req(input$chosen_scenario)
    if(input$chosen_scenario=="Fixed Time"){
      numericInput(
        inputId = ns("chosen_abs_bike"),
        label = "Bike: ",
        value = 4,
        min = 0,
        max = 200,
        step = 0.5,
        width = "100%"
      )
    }})
  
  #' Creates Low Carbon Driving Percent EV Box.
  output$percent_ev <- renderUI({
    req(input$chosen_scenario)
    if(input$chosen_scenario=="Low Carbon Driving"){
      numericInput(
        inputId = ns("chosen_pct_ev_sub"),
        label = "Percent Electric Cars",
        value = 33.5,
        min = 0,
        max = 100,
        step = 10,
        width = "100%"
      )
    }}) 
  
  
  
   
  # Input: Multiples ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates MULTIPLES bike input box.
  
  output$multiples <- renderUI({
    req(input$chosen_scenario)
    if(input$chosen_scenario=="Baseline Multiples"){
      tagList(
        splitLayout(
          numericInput(
            inputId = ns("chosen_multiples_walk"),
            label = "Walk: ",
            value = 1,
            min = 0,
            max = 5,
            step = 0.1,
            width = "100%"
          ),
          numericInput(
            inputId = ns("chosen_multiples_bike"),
            label = "Bike: ",
            value = 1,
            min = 0,
            max = 10,
            step = 0.1,
            width = "100%"
            
          ),
          numericInput(
            inputId = ns("chosen_multiples_transit"),
            label = "Transit: ",
            value = 1,
            min = 0,
            max = 4,
            step = 0.1,
            width = "100%"
          )
        ),  # end split layout
        numericInput(
          inputId = ns("chosen_car_mi_sub"),
          label = "Percent of Car Miles Substituted",
          value = 100,
          min = 0,
          max = 100,
          step = 1,
          width = "100%"
        ),
        tags$p(
          tags$em(
            "(Enter a value between 0 and 100)"
          ),
          style="color: white; text-align: center;"
        )
      )  # end of tagList
    }  # end of if statement
  })
  
  # Input: Geography Select ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates Geography selection drop down widget.
  output$geography_select <- renderUI({
    req(input$chosen_scenario)
    # req(tracker$usr_narratives)
    # If not user upload.
    if(input$chosen_scenario != "User Upload"){
      selectInput(
        inputId = ns("chosen_geography"),
        label = "Geographies (United States Only)",
        selected = "United States",
        choices = isolate_geography(),
        width="100%"
      )
    }
  })
  
  
  
  output$commentary <- renderUI({
    req(input$chosen_scenario)
    if(input$chosen_scenario == "Fixed Time"){
      tags$p(tags$em("(minutes per week)"),
             style="text-align: center; color: white;")
    }
  })
  
  # Input: Year Select ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates year selection drop down widget.
  #' 
  #' @param NA
  #' @return NA
  output$year_select <- renderUI({
    req(input$chosen_scenario)
    if(input$chosen_scenario != "User Upload" |  # If not user upload.
       input$chosen_scenario == "User Upload" & 
       !is.null(tracker$usr_narratives) 
       # is.na(tracker$usr_error)
       ){  # If user upload.
      selectInput(
        inputId = ns("chosen_year"),
        label = "Time Periods",
        selected = 2015,
        choices = seq(2015, 2050, by=5),
        width="100%"
      )}
  })
  
  # Input: Data Visualization ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates data visualization selection drop down widget.
  output$data_vis_select <- renderUI({
    req(input$chosen_scenario)
    # if(input$chosen_scenario=="User Upload"){
    #   req(tracker$usr_narratives)
    #   if(!is.null(tracker$usr_narratives)){
    #     choices <- c("Tables", "Graphs")
    #     rb <- radioButtons(inputId = ns("chosen_visual"), 
    #                        label = NA,
    #                        inline = T,
    #                        choiceNames = 
    #                          lapply(choices, 
    #                                 function(x){
    #                                   tags$p(x, 
    #                                          class = "ithim_optionButtons")
    #                                 }
    #                          ),
    #                        choiceValues = choices,
    #                        selected = "Tables")
    #   }  # end nested if statement.
    # } else {
    #   choices <- c("Report", "Infographic", "Tables", "Graphs")
    #   rb <- radioButtons(inputId = ns("chosen_visual"), 
    #                      label = NA,
    #                      inline = T,
    #                      choiceNames = 
    #                        lapply(choices, 
    #                               function(x){
    #                                 tags$p(x, 
    #                                        class = "ithim_optionButtons")
    #                               }
    #                        ),
    #                      choiceValues = choices,
    #                      selected = "Report")
    # }
    
    choices <- c("Report", "Infographic", "Tables", "Graphs")
    rb <- radioButtons(
      inputId = ns("chosen_visual"),
      label = NA,
      inline = T,
      choiceNames =
        lapply(choices,
               function(x) {
                 tags$p(x,
                        class = "ithim_optionButtons")
               }),
      choiceValues = choices,
      selected = "Report"
    )
    list(
      tags$p("Type of Output"),
      rb
    )
  })
  
  # Input: Detail Select ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates data visualization DETAIL selection drop down widget.
  output$detail_select <- renderUI({
    req(input$chosen_scenario)
    req(input$chosen_visual)
    if(input$chosen_visual %in% c("Tables", "Graphs")){
      choices <- c("Summary", "Medium", "High")
      rb <- radioButtons(inputId = ns("chosen_detail"), 
                         label = NA, 
                         inline = T,
                         choiceNames = lapply(choices, function(x){
                           tags$p(x, class = "ithim_optionButtons")
                         }),
                         choiceValues = choices,
                         selected = "Summary")
      list(
        tags$p("Level of Detail"),
        rb
      )
    }
  })
  
  
  # Input: Units - Mean/Median ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  output$units_centrality <- renderUI({
    req(input$chosen_visual)
    if(input$chosen_visual %in% c("Tables", "Graphs")){
      list(
        tags$hr(class="hr_thin"),
        tags$h4("Units", 
                class="ithimTool_modelSelection_headers"),
        tags$p("Measure of Centrality for Active Travel Time"),
        radioButtons(
          inputId = ns("chosen_units_centrality"),
          label = NA,
          inline = T,
          choiceNames = list(
            tags$p("Mean", class="ithim_optionButtons"),
            tags$p("Median", class="ithim_optionButtons")
          ),
          choiceValues=c("Mean", "Median"),
          selected = "Mean"
        )
      )
    }
  })
  
  
  # Input: Units - Active Travel Time ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates first option for units selection.
  output$units_active_travel <- renderUI({
    req(input$chosen_visual)
    if(input$chosen_visual %in% c("Tables", "Graphs")){
      list(
        tags$p(
          HTML(paste("Units for", 
                     tags$em("Active Travel Time")))
        ),
        tags$div(
          class="ithim_units_selection",
          tags$p(
            style=paste(
              "text-align: center; ",
              # "line-height: 1em; ",
              # "font-size: 1em; ",
              # "color: black; ",
              "margin: 1em; "
            ),
            "Minutes"
          ),
          tags$div(
            style=paste(
              "text-align: center; ",
              "border-top: 1px solid #4D4D4F; ",
              "margin-left: auto; ",
              "margin-right: auto; ",
              "padding-top: 0.8em; ",
              "width: 80%; "
            ),
            radioButtons(
              inputId = ns("chosen_units_active_travel_time"),
              label = NA,
              inline = T,
              choices=c("Day", "Week"),
              selected = "Week"
            ) 
          )
        )  # end of div
      )
    }
  })
  
  # Input: Units - Travel Distance ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates second option for units selection.
  output$units_travel_dist <- renderUI({
    req(input$chosen_visual)
    if(input$chosen_visual %in% c("Tables", "Graphs")){
      list(
        tags$br(),
        tags$p(
          style="margin-top: 1em; ",
          HTML(paste0(
            "Units for ",
            tags$em("Travel Distance")
          ))
        ),
        # equation box div
        tags$div(
          class="ithim_units_selection",
          tags$div(
            style = paste(
              "text-align: center; ",
              "padding: 0; ",
              "margin-left: auto; ",
              "margin-right: auto; ",
              "margin-top: 0.8em; ",
              "width: 80%; ",
              "border-bottom: 1px solid #4D4D4F; "
            ),
            radioButtons(
              inputId = ns("chosen_units_travel_dist_numerator"),
              label = NA,
              inline = T, # horizontal
              choices=c("Miles", "Kilometers"),
              selected = "Miles"
            )
          ),
          # divisor
          tags$div(
            style = paste(
              "text-align: center; ",
              "padding: 0; ",
              "margin: 0.8em; ",
              "margin-left: auto; ",
              "margin-right: auto; ",
              "width: 80%; "
            ),
            radioButtons(
              inputId = ns("chosen_units_travel_dist_denominator"),
              label = NA,
              inline = T, # horizontal,
              choices = c("Day", "Week", "Year"),
              selected = "Year"
            )
          )
        ),  # end equation box div
        tags$br()
      )  # end list
    }
  })
  
  # Input: Disease Select ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates the disease selection (conditionally) for highly detailed tables.
  output$disease_select <- renderUI({
    req(input$chosen_scenario)
    req(input$chosen_visual)
    req(tracker$specific_disease)
    if(tracker$specific_disease==T & 
       input$chosen_visual %in% c("Graphs", "Tables")) {
      selectInput(
        inputId = ns("chosen_disease"),
        label = "Choose a disease",
        choices = d$causes$ui_select,
        width = "100%"
      )
    }
  })
  
  # Print: Output Title ----
  output$title <- renderUI({
    req(input$chosen_scenario)
    req(input$chosen_visual)
    req(tracker$title_change)
    if(input$chosen_visual %in% c("Tables", "Graphs")){
      req(input$chosen_detail)
      list(
        tags$h3(
          paste0(
            input$chosen_visual,
            " (",
            input$chosen_detail,
            ")"
          ),
          class="ithimTool_h3"
        ),
        tags$hr(class="hr_dark")
      )
    } else if (input$chosen_visual %in% c("Report")) {
      list(
        tags$h3(
          paste0(
            "Summary ",
            input$chosen_visual
          ),
          class="ithimTool_h3"
        ),
        tags$hr(class="hr_dark")
      )
      
    } else {
      list(
        tags$h3(
          input$chosen_visual,
          class="ithimTool_h3"
        ),
        tags$hr(class="hr_dark")
      )
      
    }
    
  })
  
  # Print: Visualization ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates data visualization output. This is where the visualization.R is 
  #' first utilized in app.R. This renderUI function is one of the most
  #' important functions in the ITHIM TOOL, as it prepares the output of the
  #' ITHIM analytic engine in a format that user's select, including as:
  #' tables, graphs, a report, or an infographic.
  #' 
  #' @param input$chosen_scenario
  #' @param input$chosen_visual
  #' @return o, a tagList() object.
  output$vis <- renderUI({
    req(input$chosen_scenario)
    req(input$chosen_visual)
    
    # Get ITHIM model ouput 
    # (from the Analytic Engine, 
    # i.e. ithim_functions.R).
    l <- run()
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # run()'s output can be NULL if user
    # upload causes Fatal Error
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    if(is.null(l))
      return(tagList())
    
    # Here we set specific disease to NULL
    # for default for downloading csvs.
    l <- append(l, list(specific_disease=NULL))
    
    # Include the chosen_visual to l
    # (output of Analytic Engine) for
    # use in functions of 
    # visualization.R
    l <- append(l, list(visual=input$chosen_visual))
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Set Visualization Options
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Append level of detail, type of
    # visual, & disease
    if(input$chosen_visual %in% c("Tables", "Graphs")){
      req(input$chosen_detail)
      
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # TABLES & GRAPHS
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      if(input$chosen_detail=="High") {
        # Detail: HIGH
        req(input$chosen_disease)
        l <- append(l, list(detail=input$chosen_detail))
        
        # Select a specific disease. (Replaces NULL value.)
        l$specific_disease <- d$causes[
          which(d$causes$ui_select==input$chosen_disease),]
        
      } else {
        # Detail: SUMMARY & MEDIUM
        # If its Medium or Summary,
        # specific_disease is not required.
        l <- append(l, list(detail=input$chosen_detail))
      }
      
    } else {
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # INFOGRAPHIC & REPORT
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # We only need the data.frames for
      # "Summary" for either Report or
      # Infographic.
      l <- append(l, list(detail="Summary"))
    }
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Create Visuals
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    if(l$visual=="Tables") {
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # TABLE
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # obtain HTML
      o <- fn_tableManager(l)
      # Order the elements and add horizontal rules
      tmp <- list()
      # add horizontal rules/titles
      for(i in seq_along(o$tagList)) {
        tmp <- append(tmp, o$tagList[i])  # created in fn_tableTagger
        tmp <- append(tmp, list(tags$br(),
                                tags$hr(class="hr_dark")))
      }
      tmp <- append(tmp, list(tags$br()))
      o <- tmp
      rm(tmp)  # garbage collection
      
    } else if (l$visual=="Graphs") {
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # GRAPH
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # obtain data.frames for plots
      o <- fn_graphManager(l)
      
      # error checking
      if(is.null(o$objects)) {
        stop("Results of fn_graphManager is null.")
      }
      
      #TODO fix issue with sending empty df to fn_plotter
      
      # render the plots and store them in output$plot#
      for(ii in seq_along(o$objects)){
        local({
          local_i <- ii
          df <- o$objects[[local_i]]  # this is critical
          plotname <- paste("plot", 
                            local_i,
                            sep="")
          
          output[[plotname]] <- renderPlot({
            fn_plotter(df)
          })
        })  # end of local
      }  # end of loop
      
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Combine titles with tables/figures and horizontal rule.
      o <- lapply(seq_along(o$objects), function(x){
        nm <- paste("plot", x, sep="")
        out <- list()
        out <- append(out,
                      list(
                        o$titles[[x]],
                        # tags$hr(class="hr_faded"),
                        tags$div(
                          plotOutput(ns(nm),
                                     height="25em",
                                     width="100%"),
                          style=paste(
                            "-webkit-box-shadow: 0 8px 6px -6px #bfbfbf; ",
                            "-moz-box-shadow: 0 8px 6px -6px #bfbfbf; ",
                            "box-shadow: 0 8px 6px -6px #bfbfbf; "
                          )
                        )
                      )
        )
        # add horizontal rule (if not last one.)
        out <- append(out, list(
          tags$hr(class="hr_dark"),
          tags$br())
        )
        out  # return
      })
      
    } else if(l$visual=="Report"){
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # REPORT
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # Create Summary Report
      l <- fn_reporter(l)
      o <- l$report
      
    } else if(l$visual=="Infographic"){
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      # INFOGRAPHIC
      # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      local({
        output[["infographic"]] <- renderImage({
          outfile <- tempfile(pattern="ITHIM_Infographic", 
                              fileext = '.png')
          png(outfile,
              width = d$ig_w,
              height = sum(d$ig_h))
          fn_ig_pngMaker(l)
          dev.off()
          
          # Return a list containing the filename
          list(src = outfile,
               contentType = 'image/png',
               width = "100%",
               alt = "ITHIM Infographic")
          
        }, deleteFile = TRUE)
      })  # end local
      
      o <- list(
        imageOutput(ns("infographic"), 
                    height="auto", 
                    width="100%"),
        tags$br()
      )
      
    }  # end of conditional
    
    # Save for later use (possibly)
    # tracker$current_tagList <- o
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Create a value that prevents the
    # title (output$title) from changing
    # until visualization is completed.
    # Note, I'm not 100% sure this
    # actually works. Should be tested.
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    tracker$title_change <- Sys.time()
    
    # do.call(tagList,  # function tagList()
    #         o)
    
    o
  })
  
  # Print: Scenario Info ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates a box with information.
  output$info_box <- renderUI({
    req(input$chosen_scenario)
    if(input$chosen_scenario %in% d$info_box$scenario){
      list(
        tags$h4("Scenario Information",
                class="ithimTool_modelSelection_headers"),
        tags$div(
          class="ithim_info_box",
          tags$p(
            tags$b(
              d$info_box[d$info_box$scenario==input$chosen_scenario,
                         "scenario"]
            )
          ),
          HTML(
            d$info_box[d$info_box$scenario==input$chosen_scenario,
                       "info"]
          )
        ),  # end of div
        tags$hr(class="hr_thin")
      )  # end of list
    }
  })
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # DOWNLOADS ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # Button: CSV Download ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #' Creates download button.
  output$CSV_prep <- downloadHandler(
    filename = function() {
      req(input$chosen_scenario)
      paste0("ITHIM_Output-",
             input$chosen_detail,
             "_Detail_",
             input$chosen_visual,
             "s-",
             run()$baseline$name,
             "-",
             run()$scenario$name,
             "-",
             paste0(strsplit(run()$fGeography, " ")[[1]], collapse="_"),
             "_",
             run()$fYear,
             "-Generated_",
             Sys.Date(),
             ".csv")
    },
    content = function(fname) {
      req(input$chosen_scenario)
      req(input$chosen_visual)
      req(input$chosen_detail)
      req(input$chosen_year)
      
      baseline <- run()$baseline$name
      scenario <- run()$scenario$name
      generated <- Sys.Date()
      detail_level <- input$chosen_detail
      geography <- run()$fGeography
      
      if(input$chosen_detail=="High") {
        req(input$chosen_disease)
        
        # get run() output
        l <- append(run(), list(visual=input$chosen_visual, 
                                detail=input$chosen_detail,
                                clean_title=T,
                                specific_disease=NULL
        ))
        
        # retrive data.frames sans specific disease
        df_list <- fn_dataframer(l)$df_list
        
        l$specific_disease_title <- T
        
        # retrieve ALL specific disease tables
        df_list2 <- lapply(seq_len(nrow(d$causes)), function(i){
          l$specific_disease <- d$causes[i, ]
          fn_specific_disease(l)$specific_disease_df
        })

        df_list <- append(df_list, df_list2)
      } else {
        # SUMMARY AND MEDIUM
        l <- append(run(), list(visual=input$chosen_visual, 
                                detail=input$chosen_detail,
                                clean_title=T
        ))
        
        df_list <- fn_dataframer(l)$df_list
        
      }  # end conditional (high vs summary/medium)
      
      # retrieve titles.
      titles <- sapply(df_list, function(x){
        attr(x, "df_name")
      })
      
      # get ncol of biggest data.frame in df_list
      col_length <- max(sapply(df_list, ncol))
      
      # turn all column headers to a row, and rename column headers
      df_list <- lapply(df_list, function(df){
        nm <- colnames(df)
        nm <- as.data.frame(t(nm),
                            row.names=F,
                            stringsAsFactors=F)
        colnames(df) <- letters[1:ncol(df)]
        colnames(nm) <- letters[1:ncol(nm)]
        df <- rbind(nm, 
                    df,
                    stringsAsFactors=F)
      })
      
      # add padding to data.frames with less columns (for rbind)
      df_list <- lapply(df_list, function(df){
        add_length <- col_length - ncol(df)
        if(add_length==0) {
          return(df)
        } else {
          # create df of empty strings to fill in blank spots of df
          tmp <- as.data.frame(
            matrix(rep("",
                       nrow(df)*add_length),
                   ncol=add_length),
            stringsAsFactors=F)
          # add letters (eg A, B, etc) as placeholder column names for csv.
          colnames(tmp) <- letters[(ncol(df)+1):(ncol(df)+ncol(tmp))]
          df <- cbind(df, tmp)
          return(df)
        }
      })
      
      # add columns for breadcrumbs & titles of tables
      df_list <- lapply(seq_along(df_list), function(i){
        tmp <- data.frame(
          baseline = rep(baseline, nrow(df_list[[i]])),
          scenario = rep(scenario, nrow(df_list[[i]])),
          detail_level = rep(detail_level, nrow(df_list[[i]])),
          geography = rep(geography, nrow(df_list[[i]])),
          time_period = rep(input$chosen_year, nrow(df_list[[i]])),
          date_generated = rep(generated, nrow(df_list[[i]])),
          table = rep(titles[i], nrow(df_list[[i]])),
          stringsAsFactors = F
        )
        cbind(tmp, df_list[[i]])
      })
      
      
      
      df <- do.call(rbind, 
                    df_list)
      
      write.csv(df, fname,
                row.names=F)
    },
    contentType = "application/csv"
  )
  
  # Button: Download CSV ----
  output$downloadData <- renderUI({
    req(input$chosen_visual)
    if((input$chosen_visual)=="Tables") {
      tagList(
        tags$p("Download the tables as a CSV file:"),
        downloadButton(ns("CSV_prep"))
      )
    }
  })
  
  # Button: Report Download ----
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  output$Report_prep <- downloadHandler(
    filename = function() {
      req(input$chosen_scenario)
      paste0("ITHIM_Report-",
             run()$baseline$name,
             "-",
             run()$scenario$name,
             "-",
             paste0(strsplit(run()$fGeography, " ")[[1]], collapse="_"),
             "_",
             run()$fYear,
             "-Generated_",
             Sys.Date(),
             ".pdf")
    },
    content = function(file) {
      req(input$chosen_detail)
      # browser()
      tempReport <- file.path(tempdir(), "report.Rmd")
      file.copy("report.Rmd", tempReport, overwrite = TRUE)
      l <- append(run(), list(visual="Tables", 
                              detail=input$chosen_detail,
                              clean_title=T,
                              sigfigs=0
      ))
      # run through data.frame functions to get tables
      l <- fn_dataframer(l)
      df_list <- fn_formattr_df(l)$df_list
      
      # names(df_list) <- letters[1:length(df_list)]
      params <- list(df_list=df_list)
      rmarkdown::render(tempReport, 
                        output_file = file,
                        params = params,
                        envir = new.env(parent = globalenv())
      )
    },
    contentType = "application/pdf"
  )
  
  # Button: Download Report ----
  output$downloadReport <- renderUI({
    req(input$chosen_visual)
    if((input$chosen_visual)=="Tables") {
      tagList(
        # tags$p("Coming Soon: PDF Report Download"),
        # tags$p("Download a PDF Report of the Current Output:"),
        # downloadButton("Report_prep"),
        tags$hr(class="hr_thin")
      )
    }
  })
  
  
}